#ifndef _XAM_ERROR_CODES_
#define _XAM_ERROR_CODES_

#include "xam_errno.h"

extern int errno;

#define E2BIG			-2			// Argument list too long. 
#define EACCES			-3			// Permission denied. 
#define EADDRINUSE		-4			// Address in use. 
#define EADDRNOTAVAIL	-5			// Address not available. 
#define EAFNOSUPPORT	-6			// Address family not supported. 
#define EAGAIN			-7			// Resource unavailable, try again (may be the same value as [EWOULDBLOCK]). 
#define EALREADY		-8			// Connection already in progress. 
#define EBADF			-9			// Bad file descriptor. 
#define EBADMSG			-10			// Bad message. 
#define EBUSY			-11			// Device or resource busy. 
#define ECANCELED		-12			// Operation canceled. 
#define ECHILD			-13			// No child processes. 
#define ECONNABORTED	-14			// Connection aborted. 
#define ECONNREFUSED	-15			// Connection refused. 
#define ECONNRESET		-16			// Connection reset. 
#define EDEADLK			-17			// Resource deadlock would occur. 
#define EDESTADDRREQ	-18			// Destination address required. 
#define EDOM			-19			// Mathematics argument out of domain of function. 
#define EDQUOT			-20			// Reserved. 
#define EEXIST			-21			// File exists. 
#define EFAULT			-22			// Bad address. 
#define EFBIG			-23			// File too large. 
#define EHOSTUNREACH	-24			// Host is unreachable. 
#define EIDRM			-25			// Identifier removed. 
#define EILSEQ			-26			// Illegal byte sequence. 
#define EINPROGRESS		-27			// Operation in progress. 
#define EINTR			-28			// Interrupted function. 
#define EINVAL			-29			// Invalid argument. 
#define EIO				-30			// I/O error. 
#define EISCONN			-31			// Socket is connected. 
#define EISDIR			XAM_ENOTDIR			// Is a directory. 
#define ELOOP			-33			// Too many levels of symbolic links. 
#define EMFILE			-34			// Too many open files. 
#define EMLINK			-35			// Too many links. 
#define EMSGSIZE		-36			// Message too large. 
#define EMULTIHOP		-37			// Reserved. 
#define ENAMETOOLONG	-38			// Filename too long. 
#define ENETDOWN		-39			// Network is down. 
#define ENETRESET		-40			// Connection aborted by network. 
#define ENETUNREACH		-41			// Network unreachable. 
#define ENFILE			-42			// Too many files open in system. 
#define ENOBUFS			-43			// No buffer space available. 
#define ENODATA			-44			// [XSR]  No message is available on the STREAM head read queue.  
#define ENODEV			-45			// No such device. 
#define ENOENT			XAM_ENOENT			// No such file or directory. 
#define ENOEXEC			-47			// Executable file format error. 
#define ENOLCK			-48			// No locks available. 
#define ENOLINK			-49			// Reserved. 
#define ENOMEM			-50			// Not enough space. 
#define ENOMSG			-51			// No message of the desired type. 
#define ENOPROTOOPT		-52			// Protocol not available. 
#define ENOSPC			-53			// No space left on device. 
#define ENOSR			-54			// [XSR]  No STREAM resources.  
#define ENOSTR			-55			// [XSR]  Not a STREAM.  
#define ENOSYS			-56			// Function not supported. 
#define ENOTCONN		-57			// The socket is not connected. 
#define ENOTDIR			-58			// Not a directory. 
#define ENOTEMPTY		-59			// Directory not empty. 
#define ENOTSOCK		-60			// Not a socket. 
#define ENOTSUP			-61			// Not supported. 
#define ENOTTY			-62			// Inappropriate I/O control operation. 
#define ENXIO			-63			// No such device or address. 
#define EOPNOTSUPP		-64			// Operation not supported on socket. 
#define EOVERFLOW		-65			// Value too large to be stored in data type. 
#define EPERM			-66			// Operation not permitted. 
#define EPIPE			-67			// Broken pipe. 
#define EPROTO			-68			// Protocol error. 
#define EPROTONOSUPPORT	-69			// Protocol not supported. 
#define EPROTOTYPE		-70			// Protocol wrong type for socket. 
#define ERANGE			-71			// Result too large. 
#define EROFS			-72			// Read-only file system. 
#define ESPIPE			-73			// Invalid seek. 
#define ESRCH			-74			// No such process. 
#define ESTALE			-75			// Reserved. 
#define ETIME			-76			// [XSR]  Stream ioctl() timeout.  
#define ETIMEDOUT		-77			// Connection timed out. 
#define ETXTBSY			-78			// Text file busy. 
#define EWOULDBLOCK		-79			// Operation would block (may be the same value as [EAGAIN]). 
#define EXDEV			-80			// Cross-device link. 


#define XAM_EUNKNOWN             /*=*/ -1/*;*/	/* Error code is not known */
#define XAM_ENOENT               /*=*/ -2/*;*/	/* No such file or directory */
#define XAM_ESRCH                /*=*/ -3/*;*/	/* No such process */
#define XAM_EINTR                /*=*/ -4/*;*/	/* Interrupted system call */
#define XAM_EIO                  /*=*/ -5/*;*/	/* I/O error */
#define XAM_ENXIO                /*=*/ -6/*;*/	/* No such device or address */
#define XAM_E2BIG                /*=*/ -7/*;*/	/* Arg list too long */
#define XAM_ENOEXEC              /*=*/ -8/*;*/	/* Exec format error */
#define XAM_EBADF                /*=*/ -9/*;*/	/* Bad file number */
#define XAM_ECHILD               /*=*/ -10/*;*/	/* No child processes */
#define XAM_EAGAIN               /*=*/ -11/*;*/	/* Try again */
#define XAM_ENOMEM               /*=*/ -12/*;*/	/* Out of memory */
#define XAM_EACCES               /*=*/ -13/*;*/	/* Permission denied */
#define XAM_EFAULT               /*=*/ -14/*;*/	/* Bad address */
#define XAM_ENOTBLK              /*=*/ -15/*;*/	/* Block device required */
#define XAM_EBUSY                /*=*/ -16/*;*/	/* Device or resource busy */
#define XAM_EEXIST               /*=*/ -17/*;*/	/* File exists */
#define XAM_EXDEV                /*=*/ -18/*;*/	/* Cross-device link */
#define XAM_ENODEV               /*=*/ -19/*;*/	/* No such device */
#define XAM_ENOTDIR              /*=*/ -20/*;*/	/* Not a directory */
#define XAM_EISDIR               /*=*/ -21/*;*/	/* Is a directory */
#define XAM_EINVAL               /*=*/ -22/*;*/	/* Invalid argument */
#define XAM_ENFILE               /*=*/ -23/*;*/	/* File table overflow */
#define XAM_EMFILE               /*=*/ -24/*;*/	/* Too many open files */
#define XAM_ENOTTY               /*=*/ -25/*;*/	/* Not a typewriter */
#define XAM_ETXTBSY              /*=*/ -26/*;*/	/* Text file busy */
#define XAM_EFBIG                /*=*/ -27/*;*/	/* File too large */
#define XAM_ENOSPC               /*=*/ -28/*;*/	/* No space left on device */
#define XAM_ESPIPE               /*=*/ -29/*;*/	/* Illegal seek */
#define XAM_EROFS                /*=*/ -30/*;*/	/* Read-only file system */
#define XAM_EMLINK               /*=*/ -31/*;*/	/* Too many links */
#define XAM_EPIPE                /*=*/ -32/*;*/	/* Broken pipe */
#define XAM_EDOM                 /*=*/ -33/*;*/	/* Math argument out of domain of func */
#define XAM_ERANGE               /*=*/ -34/*;*/	/* Math result not representable */
#define XAM_EDEADLK              /*=*/ -35/*;*/	/* Resource deadlock would occur */
#define XAM_ENAMETOOLONG         /*=*/ -36/*;*/	/* File name too long */
#define XAM_ENOLCK               /*=*/ -37/*;*/	/* No record locks available */
#define XAM_ENOSYS               /*=*/ -38/*;*/	/* Function not implemented */
#define XAM_ENOTEMPTY            /*=*/ -39/*;*/	/* Directory not empty */
#define XAM_ELOOP                /*=*/ -40/*;*/	/* Too many symbolic links encountered */
#define XAM_EWOULDBLOCK          /*=*/ -41/*;*/	/* Operation would block */
#define XAM_ENOMSG               /*=*/ -42/*;*/	/* No message of desired type */
#define XAM_EIDRM                /*=*/ -43/*;*/	/* Identifier removed */
#define XAM_ECHRNG               /*=*/ -44/*;*/	/* Channel number out of range */
#define XAM_EL2NSYNC             /*=*/ -45/*;*/	/* Level 2 not synchronized */
#define XAM_EL3HLT               /*=*/ -46/*;*/	/* Level 3 halted */
#define XAM_EL3RST               /*=*/ -47/*;*/	/* Level 3 reset */
#define XAM_ELNRNG               /*=*/ -48/*;*/	/* Link number out of range */
#define XAM_EUNATCH              /*=*/ -49/*;*/	/* Protocol driver not attached */
#define XAM_ENOCSI               /*=*/ -50/*;*/	/* No CSI structure available */
#define XAM_EL2HLT               /*=*/ -51/*;*/	/* Level 2 halted */
#define XAM_EBADE                /*=*/ -52/*;*/	/* Invalid exchange */
#define XAM_EBADR                /*=*/ -53/*;*/	/* Invalid request descriptor */
#define XAM_EXFULL               /*=*/ -54/*;*/	/* Exchange full */
#define XAM_ENOANO               /*=*/ -55/*;*/	/* No anode */
#define XAM_EBADRQC              /*=*/ -56/*;*/	/* Invalid request code 							!!!		*/
#define XAM_EBADSLT              /*=*/ -57/*;*/	/* Invalid slot */
#define XAM_EPERM                /*=*/ -58/*;*/	/* Operation not permitted */
#define XAM_EBFONT               /*=*/ -59/*;*/	/* Bad font file format */
#define XAM_ENOSTR               /*=*/ -60/*;*/	/* Device not a stream */
#define XAM_ENODATA              /*=*/ -61/*;*/	/* No data available */
#define XAM_ETIME                /*=*/ -62/*;*/	/* Timer expired */
#define XAM_ENOSR                /*=*/ -63/*;*/	/* Out of streams resources */
#define XAM_ENONET               /*=*/ -64/*;*/	/* Machine is not on the network */
#define XAM_ENOPKG               /*=*/ -65/*;*/	/* Package not installed */
#define XAM_EREMOTE              /*=*/ -66/*;*/	/* Object is remote */
#define XAM_ENOLINK              /*=*/ -67/*;*/	/* Link has been severed */
#define XAM_EADV                 /*=*/ -68/*;*/	/* Advertise error */
#define XAM_ESRMNT               /*=*/ -69/*;*/	/* Srmount error */
#define XAM_ECOMM                /*=*/ -70/*;*/	/* Communication error on send */
#define XAM_EPROTO               /*=*/ -71/*;*/	/* Protocol error */
#define XAM_EMULTIHOP            /*=*/ -72/*;*/	/* Multihop attempted */
#define XAM_EDOTDOT              /*=*/ -73/*;*/	/* RFS specific error */
#define XAM_EBADMSG              /*=*/ -74/*;*/	/* Not a data message */
#define XAM_EOVERFLOW            /*=*/ -75/*;*/	/* Value too large for defined data type */
#define XAM_ENOTUNIQ             /*=*/ -76/*;*/	/* Name not unique on network */
#define XAM_EBADFD               /*=*/ -77/*;*/	/* File descriptor in bad state */
#define XAM_EREMCHG              /*=*/ -78/*;*/	/* Remote address changed */
#define XAM_ELIBACC              /*=*/ -79/*;*/	/* Can not access a needed shared library */
#define XAM_ELIBBAD              /*=*/ -80/*;*/	/* Accessing a corrupted shared library */

#define XAM_ELIBSCN              /*=*/ -81/*;*/	/* .lib section in a.out corrupted */
#define XAM_ELIBMAX              /*=*/ -82/*;*/	/* Attempting to link in too many shared libraries */
#define XAM_ELIBEXEC             /*=*/ -83/*;*/	/* Cannot exec a shared library directly */
#define XAM_EILSEQ               /*=*/ -84/*;*/	/* Illegal byte sequence */
#define XAM_ERESTART             /*=*/ -85/*;*/	/* Interrupted system call should be restarted */
#define XAM_ESTRPIPE             /*=*/ -86/*;*/	/* Streams pipe error */
#define XAM_EUSERS               /*=*/ -87/*;*/	/* Too many users */
#define XAM_ENOTSOCK             /*=*/ -88/*;*/	/* Socket operation on non-socket */
#define XAM_EDESTADDRREQ         /*=*/ -89/*;*/	/* Destination address required */
#define XAM_EMSGSIZE             /*=*/ -90/*;*/	/* Message too long */
#define XAM_EPROTOTYPE           /*=*/ -91/*;*/	/* Protocol wrong type for socket */
#define XAM_ENOPROTOOPT          /*=*/ -92/*;*/	/* Protocol not available */
#define XAM_EPROTONOSUPPORT      /*=*/ -93/*;*/	/* Protocol not supported */
#define XAM_ESOCKTNOSUPPORT      /*=*/ -94/*;*/	/* Socket type not supported */
#define XAM_EOPNOTSUPP           /*=*/ -95/*;*/	/* Operation not supported on transport endpoint */
#define XAM_EPFNOSUPPORT         /*=*/ -96/*;*/	/* Protocol family not supported */
#define XAM_EAFNOSUPPORT         /*=*/ -97/*;*/	/* Address family not supported by protocol */
#define XAM_EADDRINUSE           /*=*/ -98/*;*/	/* Address already in use */
#define XAM_EADDRNOTAVAIL        /*=*/ -99/*;*/	/* Cannot assign requested address */

#define XAM_ENETDOWN             /*=*/ -100/*;*/	/* Network is down */
#define XAM_ENETUNREACH          /*=*/ -101/*;*/	/* Network is unreachable */
#define XAM_ENETRESET            /*=*/ -102/*;*/	/* Network dropped connection because of reset */
#define XAM_ECONNABORTED         /*=*/ -103/*;*/	/* Software caused connection abort */
#define XAM_ECONNRESET           /*=*/ -104/*;*/	/* Connection reset by peer */
#define XAM_ENOBUFS              /*=*/ -105/*;*/	/* No buffer space available */
#define XAM_EISCONN              /*=*/ -106/*;*/	/* Transport endpoint is already connected */
#define XAM_ENOTCONN             /*=*/ -107/*;*/	/* Transport endpoint is not connected */
#define XAM_ESHUTDOWN            /*=*/ -108/*;*/	/* Cannot send after transport endpoint shutdown */
#define XAM_ETOOMANYREFS         /*=*/ -109/*;*/	/* Too many references: cannot splice */
#define XAM_ETIMEDOUT            /*=*/ -110/*;*/	/* Connection timed out */
#define XAM_ECONNREFUSED         /*=*/ -111/*;*/	/* Connection refused */
#define XAM_EHOSTDOWN            /*=*/ -112/*;*/	/* Host is down */
#define XAM_EHOSTUNREACH         /*=*/ -113/*;*/	/* No route to host */
#define XAM_EALREADY             /*=*/ -114/*;*/	/* Operation already in progress */
#define XAM_EINPROGRESS          /*=*/ -115/*;*/	/* Operation now in progress */
#define XAM_ESTALE               /*=*/ -116/*;*/	/* Stale NFS file handle */
#define XAM_EUCLEAN              /*=*/ -117/*;*/	/* Structure needs cleaning */
#define XAM_ENOTNAM              /*=*/ -118/*;*/	/* Not a XENIX named type file */
#define XAM_ENAVAIL              /*=*/ -119/*;*/	/* No XENIX semaphores available */
#define XAM_EISNAM               /*=*/ -120/*;*/	/* Is a named type file */
#define XAM_EREMOTEIO            /*=*/ -121/*;*/	/* Remote I/O error */
#define XAM_EDQUOT               /*=*/ -122/*;*/	/* Quota exceeded */
#define XAM_ENOMEDIUM            /*=*/ -123/*;*/	/* No medium found */
#define XAM_EMEDIUMTYPE          /*=*/ -124/*;*/	/* Wrong medium type */

#endif

