#ifndef _LINE_DISCIPLINE_H_
#define _LINE_DISCIPLINE_H_

/*9999999999999999999999999999999999999999 
	Berkeley's user's space includes
9999999999999999999999999999999999999999 */
#include "sys/termios.h"
#include "sys/ttycom.h"

#include <sys/fcntl.h>

static const int nInputBufferSize = 512;
static const int nOutputBufferSize = 4096;


typedef enum {
	NonBlockingMode,
	CanonicalMode,
	MinimalCharMode,
	TimedCharMode,
	TimedMode,
	RawMode
} typeReadMode;

class LineDiscipline
{
	pid_t						m_OwnerProcess;
	// a read syscall related info
	typeReadMode				m_nReadMode;
	int							m_nLinesDetected;
	int							m_nReadBytes;
	int							m_nReadResult;
	// input biffer descriptor
	char						InputBuffer[nInputBufferSize];
	int							nInputBytesCount;
	// a write syscall related info
	bool						m_bStopOutput;
	char						OutputBuffer[nOutputBufferSize];
	int							nOutputBytesCount;
	int							nOutputBufferBeginIndex;
	int							nOutputBufferEndIndex;
	// parsed open attributes
	bool						bNonBlock;
	bool						bEOF;
	// termios attributes
	struct termios				Discipline;
protected:
	bool PutByteIntoOutputBuffer( char ch );
	bool GetByteFromOutputBuffer( char * ch );

protected:	// Fllowing methods must be defined in inherited class
	virtual int	ConfirmRead( void * pBuffer, int nBytesRead ) = 0;
	virtual int	ConfirmWrite( int nBytesRead ) = 0;
	virtual int		SendData( bool bConfirm ) = 0;

protected:
	bool	IsControlChar( char ch, int & nIdx );
	int		CopyOut( int nBytesProceed );
	int		CanonicalCopyOut( void );
	void	SetReadMode( void );
	void	OnTimeout( void );
	bool	ReadBuffer( void );
	void	CompleteRead(  int nBytesRead  );

public:
	int		Initialize();
	int		ReceiveData( void * pBuffer, int nBytes );
	//		Syscall handlers
	int		Open(int nFlags, ... );
	int		Close();
	int		Read( int nBytes );
	void	Write( void * pBuffer, int nBytes  );
	//		IOCTL helpers
	int		StopOutput();
	int		StartOutput();
	int		Flush( int nFlushMode );
	int		GetTermiosStruct( struct termios * ts );
	int		SetTermiosStruct( struct termios * ts );
	pid_t	GetOwnerProcess(void) { return m_OwnerProcess; }
	void	SetOwnerProcess(pid_t tid) { m_OwnerProcess = tid; }

	LineDiscipline();
	virtual ~LineDiscipline();
};

#endif
