#ifndef _SIGNAL_H_
#define _SIGNAL_H_

#include <pthread.h>
#include <sys/cdefs.h>
#include <sys/types.h>


#define SI_USER			0 /* The signal was sent by the kill() function. The implementation may set si_code to SI_USER if the signal was sent by the raise() or abort() functions or any similar functions provided as implementation extensions. */
#define SI_QUEUE		1 /* The signal was sent by the sigqueue() function. */
#define SI_TIMER		2 /* The signal was generated by the expiration of a timer set by timer_settime(). */
#define SI_ASYNCIO		3 /* The signal was generated by the completion of an asynchronous I/O request. */
#define SI_MESGQ		4 /* The signal was generated by the arrival of a message on an empty message queue. */
#define SI_SUPERVISOR	5




#define NSIG		32

#define	SIGHUP		1	/* Hangup (POSIX).  */
#define	SIGINT		2	/* Interrupt (ANSI).  */
#define	SIGQUIT		3	/* Quit (POSIX).  */
#define	SIGILL		4	/* Illegal instruction (ANSI).  */
#define	SIGTRAP		5	/* Trace trap (POSIX).  */
#define	SIGABRT		6	/* Abort (ANSI).  */
#define	SIGIOT		6	/* IOT trap (4.2 BSD).  */
#define	SIGBUS		7	/* BUS error (4.2 BSD).  */
#define	SIGFPE		8	/* Floating-point exception (ANSI).  */
#define	SIGKILL		9	/* Kill, unblockable (POSIX).  */
#define	SIGUSR1		10	/* User-defined signal 1 (POSIX).  */
#define	SIGSEGV		11	/* Segmentation violation (ANSI).  */
#define	SIGUSR2		12	/* User-defined signal 2 (POSIX).  */
#define	SIGPIPE		13	/* Broken pipe (POSIX).  */
#define	SIGALRM		14	/* Alarm clock (POSIX).  */
#define	SIGTERM		15	/* Termination (ANSI).  */
#define	SIGSTKFLT	16	/* Stack fault.  */
#define	SIGCLD		SIGCHLD	/* Same as SIGCHLD (System V).  */
#define	SIGCHLD		17	/* Child status has changed (POSIX).  */
#define	SIGCONT		18	/* Continue (POSIX).  */
#define	SIGSTOP		19	/* Stop, unblockable (POSIX).  */
#define	SIGTSTP		20	/* Keyboard stop (POSIX).  */
#define	SIGTTIN		21	/* Background read from tty (POSIX).  */
#define	SIGTTOU		22	/* Background write to tty (POSIX).  */
#define	SIGURG		23	/* Urgent condition on socket (4.2 BSD).  */
#define	SIGXCPU		24	/* CPU limit exceeded (4.2 BSD).  */
#define	SIGXFSZ		25	/* File size limit exceeded (4.2 BSD).  */
#define	SIGVTALRM	26	/* Virtual alarm clock (4.2 BSD).  */
#define	SIGPROF		27	/* Profiling alarm clock (4.2 BSD).  */
#define	SIGWINCH	28	/* Window size change (4.3 BSD, Sun).  */
#define	SIGPOLL		SIGIO	/* Pollable event occurred (System V).  */
#define	SIGIO		29	/* I/O now possible (4.2 BSD).  */
#define	SIGPWR		30	/* Power failure restart (System V).  */
#define SIGSYS		31	/* Bad system call.  */
#define SIGUNUSED	31

//typedef int sig_atomic_t;	// Possibly volatile-qualified integer type of an object that can be accessed 
									// as an atomic entity, even in the presence of asynchronous interrupts. 
#pragma pack(4)

union sigval
{
	int    sival_int;    // Integer signal value. 
	void  *sival_ptr;    // Pointer signal value. 
};

typedef struct siginfo_t
{
	int           si_signo;  // Signal number. 
	int           si_code;   // Signal code. 
/*
	int           si_errno;  // If non-zero, an errno value associated with this signal, as defined in <errno.h>. 
	pid_t         si_pid;    // Sending process ID. 
	uid_t         si_uid;    // Real user ID of sending process. 
	void         *si_addr;   // Address of faulting instruction. 
	int           si_status; // Exit value or signal. 
	long          si_band;   // Band event for SIGPOLL. 
*/
	union sigval  si_value;  // Signal value. 
} siginfo_t;

#pragma pack(0)


typedef void (*sighandler_t)(int);
typedef void (*sa_sigaction_t)(int, siginfo_t *, void *);

typedef struct sigevent
{
	int						sigev_notify;            // Notification type. 
	int						sigev_signo;             // Signal number. 
	union sigval			sigev_value;             // Signal value. 
	void					(*sigev_notify_function)(union sigval);   // Notification function. 
	pthread_attr_t	*		sigev_notify_attributes     ; // Notification attributes. 

} sigevent_t;

typedef int sigset_t;
typedef int sig_atomic_t;

typedef struct sigaction
{
	union
	{
		void (*sa_handler)(int);  // Pointer to a signal-catching function or one of the macros SIG_IGN or SIG_DFL. 
		void (*sa_sigaction)(int, siginfo_t *, void *);	// Pointer to a signal-catching function. 
	};
	sigset_t sa_mask;         // Set of signals to be blocked during execution of the signal handling function. 
	int      sa_flags;        // Special flags. 
} sigaction_t;




//#define SIG_ERR	((sighandler_t) -1)		/* Error return.  */
//#define SIG_DFL	((sighandler_t) 0)		/* Default action.  */
//#define SIG_IGN	((sighandler_t) 1)		/* Ignore signal.  */

#define SIG_STP	((sighandler_t) -3)		/* Ignore signal.  */
#define SIG_CNT	((sighandler_t) -2)		/* Ignore signal.  */
#define SIG_ERR	((sighandler_t)	-1)		/* Error return.  */
#define SIG_DFL	((sighandler_t) 0)		/* Default action.  */
#define SIG_IGN	((sighandler_t) 1)		/* Ignore signal.  */
#define SIG_TRM	((sighandler_t) 2)		/* Ignore signal.  */
#define SIG_COR	((sighandler_t) 3)		/* Ignore signal.  */

#define _SIG_STP_	-3		/* Ignore signal.  */
#define _SIG_CNT_	-2		/* Ignore signal.  */
#define _SIG_ERR_	-1		/* Error return.  */
#define _SIG_DFL_	0		/* Default action.  */
#define _SIG_IGN_	1		/* Ignore signal.  */
#define _SIG_TRM_	2		/* Ignore signal.  */
#define _SIG_COR_	3		/* Ignore signal.  */

#define SIG_BLOCK	0
#define SIG_UNBLOCK	1
#define SIG_SETMASK	2

#define sig_t	sighandler_t

__BEGIN_DECLS

	sighandler_t	signal(int signum, sighandler_t handler);
	int				raise(int sig);
	int				sigaction(int, const struct sigaction *in, struct sigaction *out);
	int				sigfillset(sigset_t *);
	int				sigprocmask(int, const sigset_t *in, sigset_t *out);
	int 			kill(pid_t pid, int sig);
	int				sigqueue(pid_t pid, int sig, union sigval value);
	
    int sigemptyset(sigset_t *set);
    int sigfillset(sigset_t *set);
    int sigaddset(sigset_t *set, int signo);
    int sigdelset(sigset_t *set, int signo);
    int sigismember(const sigset_t *set, int signo);
    
__END_DECLS

#endif
