#ifndef _STDLIB_H_
#define _STDLIB_H_

#include <sys/cdefs.h>
#include <ctype.h>
#include <sys/types.h>

# if 1

	__BEGIN_DECLS

	int atoi(const char * str);
	long atol(const char * str);
	long strtol(const char *p, char **out_p, int base);

#define strtoul	(unsigned long int) strtol

	int setenv(const char * name, const char * value, int overwrite);
	int unsetenv(const char * name);
	char * getenv(const char * name);
	int putenv(char * string);

	void qsort(void	* base, size_t nmemb, size_t size, int(*compar)(const void *, const void *));
	void * bsearch(	const void * key, const void * base, size_t nmemb, size_t size, int (*compar) (const void *, const void *));

	// Memory allocation functions
	void *calloc(size_t nmemb, size_t size);
	void *malloc(size_t size);
	void free(void *ptr);
	void *realloc(void *ptr, size_t size);

 	void exit(int error_code);
	void abort(void);
	int abs(int v);
 
	int mkstemp( char * templat );
	int mktemp( char * tmplate );

	__END_DECLS

# else

static inline int atoi(const char * str)
{
	int	result	= 0;
	int	multi = 10;
	int	sign = 1;
	
	if(*str == '-') { sign  = -1;  str++; } else if(*str=='+') str++;

	while( isdigit(*str) )
	{
		result = (*str - 0x30) + result * multi;
		str++;
	}
	return sign * result;
}

# endif


#endif
