#ifndef _STAT_H_
#define _STAT_H_

#include <sys/cdefs.h>
#include <sys/types.h>

#define S_IFMT		0170000	///< type of file
#define S_IFREG 	0100000		///< regular
#define S_IFRPC		0160000		///< symlink to file
#define S_IFSOCK	0140000		///< symlink to file
#define S_IFLNK		0120000		///< symlink to file
#define S_IFBLK		0060000		///< block special
#define S_IFDIR		0040000  	///< directory
#define S_IFCHR		0020000		///< character special
#define S_IFIFO		0010000		///< this is a FIFO
#define S_ISUID		0004000		///< set user id on execution
#define S_ISGID		0002000		///< set group id on execution
#define S_ISVTX		0001000		///< sticky bit
//#define S_ISTXT		S_ISVTX
#define S_ISTXT		0001000

/** next is reserved for future use */

#define S_ISVTX 0001000		///< save swapped text even after use */

/*! 
	\def
	POSIX masks for st_mode. 
*/
#define S_IRWXU   00700		///< owner:  rwx------
#define S_IRUSR   00400		///< owner:  r--------
#define S_IWUSR   00200		///< owner:  -w-------
#define S_IXUSR   00100		///< owner:  --x------

#define S_IRWXG   00070		///< group:  ---rwx---
#define S_IRGRP   00040		///< group:  ---r-----
#define S_IWGRP   00020		///< group:  ----w----
#define S_IXGRP   00010		///< group:  -----x---

#define S_IRWXO   00007		///< others: ------rwx
#define S_IROTH   00004		///< others: ------r--
#define S_IWOTH   00002		///< others: -------w-
#define S_IXOTH   00001		///< others: --------x

/*! 
	\def
	The following macros test st_mode (from POSIX Sec. 5.6.1.1). 
*/
#define S_ISREG(m)	(((m) & S_IFMT) == S_IFREG)	///< is a reg file
#define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)	///< is a directory
#define S_ISCHR(m)	(((m) & S_IFMT) == S_IFCHR)	///< is a char spec
#define S_ISBLK(m)	(((m) & S_IFMT) == S_IFBLK)	///< is a block spec
#define S_ISFIFO(m)	(((m) & S_IFMT) == S_IFIFO)	///< is a pipe/FIFO
#define S_ISSOCK(m)	(((m) & S_IFMT) == S_IFSOCK)///< is a socket
#define S_ISLNK(m)	(((m) & S_IFMT) == S_IFLNK)	///< is a symlink
#define S_ISSTICTKY(m)		((m) & S_ISVTX)		///< is a sticky executable



__BEGIN_DECLS

    mode_t umask(mode_t mask);
    int stat(const char *file_name, struct stat *buf);
    int lstat(const char *file_name, struct stat *buf);
    int fstat(int fd, struct stat *buf);

__END_DECLS

#endif
