#ifndef _XAM_DRIVER_
#define _XAM_DRIVER_

#include <l4/kip.h>

#if defined (WIN32)

typedef enum
{
	category_RawDeviceService,
	category_StreamDeviceService,
	category_BlockDeviceService,
	category_ProtocolService
} typeDriverCategory;

typedef enum
{
	drivermodel_Module,
	drivermodel_Process,
	drivermodel_Thread
} typeDriverModel;

#else
# include "sys/device.h"
#endif

#pragma pack(1)

typedef struct 
{
	L4_Word_t				pIP;				// Code counter
	L4_Word_t				TextStart;			// Text segment start
	L4_Word_t				TextSize;			// Text size
	L4_Word_t				DataStart;			// Data segment start
	L4_Word_t				DataSize;			// Data segment size
	L4_Word_t				BssStart;			// BSS segment start
	L4_Word_t				BssSize;			// BSS segment size
	L4_Word_t				StackBottom;		// Stack pointer
	L4_Word_t				StackSize;			// Stack size
} ParsedBinaryImage;

typedef struct 
{
	unsigned int			ProtocolVersion;	// Version of the Driver Startup protocol 
	typeDriverCategory		Category;			// Describes driver type: device or protocol
	typeDriverModel			Model;				// Kernel module, standalone process, or thread

	L4_Word_t				Minor;				// Minor device number
	unsigned int			MinorCount;			// Count of minor devices

	union	
	{
		ParsedBinaryImage	process_image;
		L4_ThreadId_t		thread;
		void			*	null;
	};


	int						MaxHeapSize;		// Maximum amount of dynamic aloocated memory
	char					DeviceName[16];		// Device name (not null terminated)
	unsigned long			MagicLabel;			// Dedicated label 0xfeed0dad used to resist request 

} DeviceDriverRegistrationReq_t;

#pragma pack()

const unsigned long		MagicLabelValue = 0xfeedface;

extern "C" L4_MsgTag_t xam_service_registration_request(
	const L4_Word_t								handle,
    const DeviceDriverRegistrationReq_t		*	req, 
    const char								*	arguments );

extern "C" int xam_confirm_service_startup( void );

extern "C" L4_MsgTag_t xam_service_shutodwn_request(
	const int									reason,
	const DeviceDriverRegistrationReq_t		*	req );


#endif
