#ifndef _XAM_REQUESTS_H
#define _XAM_REQUESTS_H

#include <l4/types.h>
#include <l4/ipc.h>
#include <sys/types.h>

typedef enum 
{
// New order
	cmdZero					= 0x00,
	cmdAllocateSegment		= 0x01,
	cmdReleaseSegment		= 0x02,
	cmdProcessCreate		= 0x03,
	cmdProcessDestroy		= 0x04,
	cmdThreadCreate			= 0x05,
	cmdThreadDestroy		= 0x06,
	cmdProcessFork			= 0x07,
	cmdExecImage			= 0x08,
	cmdDeviceStart			= 0x09,
	cmdDeviceStop			= 0x0a,
	cmdEnumerateDevice		= 0x0b,
	cmdGetDeviceHandle		= 0x0c,
	cmdGetProcessID			= 0x0d,
	cmdGetThreadID			= 0x0e,
	cmdChangeHeapSize		= 0x0f,
	cmdSetSignal			= 0x10,
	cmdSendSignal			= 0x11,
	cmdWaitProcess			= 0x12,
	cmdSetTimer				= 0x13,
	cmdSetNotification		= 0x14,

//----
	cmdSystemHalt			= 0x15,
	cmdLeaveSignal			= 0x16,
	cmdStartupIPC			= 0x17,
	cmdGetBootClock			= 0x18,
} RootTaskCommand;

typedef enum 
{
	fsMount					=	0x40,
	fsUnMount				= 	0x41,
	fsGetCurrentDirectory	= 	0x42,
	fsChangeDir				=	0x43,
	fsOpenDirectory			=	0x44,
	fsReadDirectory			=	0x45,
	fsCloseDirectory		=	0x46,
	fsGetFileStatus			=	0x47,
	fsChangeRoorDirectory	=	0x48,
	fsOpenFile				=	0x49,
	fsCloseFile				=	0x4a,
	fsReadFile				=	0x4b,
	fsWriteFile				=	0x4c,
	fsRemoveFile			=	0x4d,
	fsRemoveDirectory		=	0x4e,
	fsCreateDirectory		=	0x4f,
	fsCreateHardLink		=	0x50,
	fsCreateSybolicLink		=	0x51,
	fsRenameFile			=	0x52,
	fsSeekFile				=	0x53,
	fsGetSuperblockStatus	=	0x54,
	fsChangeOwner			=	0x55,
	fsChangeMode			=	0x56,
	fsFlushCaches			=	0x57,
	fsDupDescriptor			=	0x58,
	fsCopyDescriptor		=	0x59,
	fsCreatePipe			=	0x5a,
	fsForkProcess			=	0x5b,
	fsExec					=	0x5c,
//	fsGetEnvironment		=	0x5d,
//	fsSetEnvironment		=	0x5e,
//	fsUnsetEnvironment		=	0x5f,
	fsMakeNode				=	0x60,
	fsIOCTL					=	0x61,
	fsExit					=	0x62,
	fsFileControl			=	0x63,
	fsChangeFileTimes		=	0x64,
	fsCreateSession			=	0x65,
} FileServiceCommand;

typedef enum {
	RpcDevice_Read			=	0x6f
} RpcServiceCommand;


typedef enum                            
{                                       
    StreamDevice_ConfirmSend	= 0x80,    
    StreamDevice_ReceiveData	= 0x81,  
	StreamDevice_Open			= 0x82,
	StreamDevice_Close			= 0x83,
    StreamDevice_Read			= 0x84, 
    StreamDevice_Write			= 0x85,
	StreamDevice_IOCTL			= 0x86,
	StreamDevice_Log			= 0x87,

} enumStreamDeviceLabels;                 

typedef enum
{
	BlockDevice_Open			= 0x90,
	BlockDevice_Close			= 0x91,
	BlockDevice_Read			= 0x92,
	BlockDevice_Write			= 0x93,
	BlockDevice_Flush			= 0x94,
	BlockDevice_IOCTL			= 0x95
} enumBlockDevieLabels;


/* Filesystem section */
L4_MsgTag_t 	XAM_FileSystemCall( 
	L4_Msg_t		*	msg, 
	const char		*	szDebugMessage = "");

/* Supervisor section */
extern L4_ThreadId_t GetDeviceHandle(const char * szDeviceName);
extern int RequestSendSignal(
	L4_ThreadId_t		tid,
	int					nSignal,
	L4_Word_t			nSignalAttributes,
	L4_Word_t			nSignalValue );

extern int RequestThreadDestroy(L4_ThreadId_t	tid);
extern int RequestProcessDestroy(L4_ThreadId_t	tid);
extern int RequestProcessCreate(
	L4_Word_t				handle,
	L4_Word_t				ip,
	L4_Word_t				text_vstart,
	L4_Word_t				text_size,
	L4_Word_t				data_vstart,
	L4_Word_t				data_size,
	L4_Word_t				bss_vstart,
	L4_Word_t				bss_size,
	L4_StringItem_t			command,
	L4_StringItem_t			environment,
	L4_ThreadId_t		*	tid,
	L4_Word_t			*	pid = 0);


extern "C" int			map_region(void * address, int size, void * map_to, int * handle );
extern "C" int			allocate_segment(void * address, int size, void * map_to, int * handle );
extern "C" int			release_segment(int handle, int * ref_cnt);

extern "C" int sys_gettidbypid( 
	int					pid, 
	L4_ThreadId_t	*	tid );

extern "C" int 	sys_getpidbytid( 
	L4_ThreadId_t 		tid,  
	pid_t 			*	pid, 
	pid_t 			*	ppid );

extern "C" int sys_fork(
	L4_ThreadId_t		parent, 
	L4_ThreadId_t	*	child_tid,
	L4_Word_t		*	child_pid);

extern "C" int sys_sigqueue (
	L4_ThreadId_t			tid, 
	int						signal,
	int						attrib,
	union sigval			value );

extern "C" int sys_exec(
	int					tid,
	int					nHandle,
	int					ip,
	int					text_vstart,
	int					text_size,
	int					data_vstart,
	int					data_size,
	int					bss_vstart,
	int					bss_size,
	L4_StringItem_t	*	arguments);


#endif
