#include <l4/ipc.h>
#include <l4/kip.h>
//#include <l4io.h>

#include <l4/kdebug.h>

#include <stub.h>
#include <string.h>
#include <xam_errno.h>
#include <sys/xam_driver.h>
#include <sys/xam_blkdevice.h>
#include <sys/xam_requests.h>
#include <sys/xam_console.h>

#include "ramdisk_driver.h"

extern "C" void _driver_entry (void);

//extern void * _stack;

extern unsigned char		*	_binary_HDA_img_start;
extern unsigned char		*	_binary_HDA_img_end;
extern int						_binary_HDA_img_size;

static char						RcvBuffer[4096];
static L4_ThreadId_t			Owner;


#if 1

const	DeviceDriverRegistrationReq_t		registration_request = 
{
	1,
	category_BlockDeviceService,
	drivermodel_Thread,
	0, // Minor 
	1, // MinorCount 
	{ 
		{ (L4_Word_t) _driver_entry, 0, 0, 0, 0, 0, 0, 0, 8192 } 
	},
	0, // MaxHeapSize 
	{ 'i','n','i','t','r','d', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	MagicLabelValue
};

#else

extern "C" void _driver_entry	(void);
extern "C" void _text_start		(void);
extern "C" void _text_size		(void);
extern "C" void _data_start		(void);
extern "C" void _data_size		(void);
extern "C" void __bss_start		(void);
extern "C" void __bss_size		(void);

const	DeviceDriverRegistrationReq_t		registration_request = 
{
	1,		
	category_BlockDeviceService,
	drivermodel_Process,
	0, // Minor 
	1, // MinorCount 
	{ 
		{ 
			(L4_Word_t) _driver_entry, 
			(L4_Word_t) _text_start, (L4_Word_t) _text_size, 
			(L4_Word_t) _data_start, (L4_Word_t) _data_size, 
			(L4_Word_t) __bss_start, (L4_Word_t) __bss_size, 
			0, 8192 
		} 
	},
	0, // MaxHeapSize 
	{ 'i','n','i','t','r','d', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	MagicLabelValue
};

#endif

extern "C" int register_service( 
	L4_Word_t					handle, 
	char					*	arguments )
{
	L4_Word_t					nStatus = 0;
	L4_MsgTag_t					tag;

	tag = xam_service_registration_request( handle, &registration_request, arguments );
	if (L4_IpcFailed(tag)) 
	{
		L4_KDB_Enter( "RAMDISK registration IPC failed" );
		nStatus = L4_ErrorCode();
	}
	_driver_entry();

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
L4_ThreadId_t Open(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_ThreadId_t	Own;

	L4_Word_t	nContext = L4_Get (msg, 0);
	L4_Word_t	nMinor = L4_Get (msg, 1);
		
	Own.raw = L4_Get (msg, 2);
	
	L4_Clear(msg);
	L4_Set_Label(msg, BlockDevice_Open);

	L4_Append(msg, nContext);		// Context

	if( L4_IsNilThread( Owner ) )
	{
		L4_Append(msg, 0);				// Status
		L4_Append(msg, 0x80);			// Type
		Owner = Own;
	}
	else
	{
		L4_Append(msg, XAM_EBUSY);		// Status
		L4_Append(msg, 0);				// Type
	}

	xam_printf( 0, "Virtual disk image size is %d bytes\n", (int) &_binary_HDA_img_size);

	return Owner;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
L4_ThreadId_t  ReadBlock(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	int							nStatus;
	unsigned int				nBlockNumber, nBlockSize, nPosition;
	L4_Word_t					nContext;
	L4_Word_t					nDeviceNo;
	L4_Word_t					nCacheControlWord;
	L4_StringItem_t				Block;
	
	do {
		nContext			= L4_Get (msg, 0);

		if(L4_UntypedWords (tag) != 5 && L4_TypedWords (tag) != 0) 		
		{
			xam_printf( 0, "RD - request 'ReadBlock' format error\n" );
			nStatus = XAM_EINVAL;
			break;
		}
		nDeviceNo			= L4_Get (msg, 1);
		nBlockNumber		= L4_Get (msg, 2);
		nBlockSize			= L4_Get (msg, 3);
		nCacheControlWord	= L4_Get (msg, 4);
		
		Block = L4_StringItem ( 0, (void*) 0 );
		nPosition = nBlockNumber * nBlockSize;
		
		if(nPosition + nBlockSize > (unsigned int) &_binary_HDA_img_size)
		{
			xam_printf( 3, "RD - request argument error\n" );
			nStatus = XAM_ESPIPE;
			break;
		}

		char	*	disk = (char*) &_binary_HDA_img_start;
        Block = L4_StringItem ( nBlockSize, (void*) &disk[nPosition] );
		nStatus = nBlockSize;
		
	} while(false);

	L4_Clear(msg);
    L4_Set_Label(msg, BlockDevice_Read);
	L4_Append(msg, nContext );
	L4_Append(msg, nStatus );
	L4_Append(msg, &Block);
	return Owner;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
L4_ThreadId_t  WriteBlock(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	int							Status;

	unsigned int				nBlockNumber;
	unsigned int				nBlockSize;
	L4_Word_t					nContext;
	L4_Word_t					nDeviceNo;
	L4_Word_t					nCacheControlWord;

	unsigned int				nPosition;

	do {
	
		nContext			= L4_Get (msg, 0);

		if(L4_UntypedWords (tag) != 5) 
		{
			xam_printf(1, "RD - request 'WriteBlock' format error\n" );
			Status = XAM_EINVAL;
			break;
		}

		nDeviceNo			= L4_Get (msg, 1);
		nBlockNumber		= L4_Get (msg, 2);
		nBlockSize			= L4_Get (msg, 3);
		nCacheControlWord	= L4_Get (msg, 4);

		nPosition = nBlockNumber * nBlockSize;
		
		if(nPosition + nBlockSize > (unsigned int) &_binary_HDA_img_size)
		{
			xam_printf(3, "Request out of range : %d\n", nPosition );
			Status = XAM_ESPIPE;
			break;
		}
		
		char	*	disk = (char*) &_binary_HDA_img_start;

		memcpy( &disk[nPosition], RcvBuffer, nBlockSize );

		Status = nBlockSize;

	} while(false);

	L4_Clear(msg);
    L4_Set_Label(msg, BlockDevice_Write);
	L4_Append(msg, nContext);	// Context
	L4_Append(msg, Status);

	//xam_printf(1, "RamDisk - WRITE_BLOCK [%d] request %d bytes", nBlockNumber, nBlockSize );
	//for(unsigned int i = 0; i < L4_UntypedWords (L4_MsgTag(msg)) + L4_TypedWords (L4_MsgTag(msg));  i++)
	//{
	//	xam_printf(1, "%p  ", (void *) L4_Get (msg, i) );
	//}
	//xam_printf(1, "\n" );
	
	return Owner;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
L4_ThreadId_t  DeviceFlush(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_Word_t					nContext;
	L4_Word_t					nDeviceNo;

	nContext			= L4_Get (msg, 0);
	nDeviceNo			= L4_Get (msg, 1);

	xam_printf( 3, "RAMDISK: Got flush request\n" );

	L4_Clear(msg);
    L4_Set_Label(msg, BlockDevice_Flush );
	L4_Append(msg, nContext);		// Context
	L4_Append(msg, 0 );				// Status OK

	return Owner;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
L4_ThreadId_t  DeviceIOCTL(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	L4_Word_t					nContext;
	L4_Word_t					nDeviceNo;

	nContext			= L4_Get (msg, 0);
	nDeviceNo			= L4_Get (msg, 1);

	xam_printf( 3, "RAMDISK: Got IOCTL request\n" );

	L4_Clear(msg);
    L4_Set_Label(msg, BlockDevice_IOCTL );
	L4_Append(msg, nContext);		// Context
	L4_Append(msg, XAM_ENOSYS );	// Status OK

	return Owner;
}


///////////////////////////////////////////////////////////////////////////////
//
//
//
void UnknownRequest(
	L4_MsgTag_t					tag,
	L4_Msg_t				*	msg)
{
	xam_printf (3, "RAMDISK DRIVER: got unknonw msg [0x%08lx] (%p, %p) from %p\n", 
		(long) L4_Label (tag),
		(void *) L4_Get (msg, 0), 
		(void *) L4_Get (msg, 1),
		(void *) 0 /*TID.raw*/);
	
#define L4_REQUEST_MASK         ( ~((~0UL) >> ((sizeof (L4_Word_t) * 8) - 20)))

	xam_printf(3, "   ->[0x%08lx] Untyped words %lx Typed words %lx\n",
		tag.raw & L4_REQUEST_MASK,
		L4_UntypedWords (tag),
		L4_TypedWords (tag) );
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
extern "C" void driver_main (int argc, char * argv[] )                                           
{
	L4_ThreadId_t				TID;
	L4_MsgTag_t					tag;
	L4_Msg_t					msg;
	L4_MsgBuffer_t				MsgBuff;
	L4_StringItem_t 			str;

	xam_confirm_service_startup();
	xam_console_init("vty");
	xam_printf(0,  "Ramdisk ready\n" );

	str = L4_StringItem (sizeof(RcvBuffer), (void*) RcvBuffer);

	L4_Clear(&MsgBuff);
	L4_Append(&MsgBuff, str);
	L4_Accept( L4_StringItemsAcceptor, &MsgBuff );

	// Server loop
	while(true)
	{
	
		tag = L4_Wait (&TID);
		if (L4_IpcFailed (tag))
		{
			xam_printf(3, "RAMDISK: IPC failed : Error code 0x%lx\n", L4_ErrorCode() );
			continue;
		}
		L4_Store (tag, &msg);

		switch( L4_Label(tag) & 0xff )
		{

		case BlockDevice_Open:
			TID = Open( tag, &msg );
			break;

		case BlockDevice_Close:
			break;

		case BlockDevice_Read:
			TID = ReadBlock( tag, &msg );
			break;

		case BlockDevice_Write:
			TID = WriteBlock( tag, &msg );
			break;

		case BlockDevice_Flush:
			TID = DeviceFlush( tag, &msg );
			break;

		case BlockDevice_IOCTL:
			TID = DeviceIOCTL( tag, &msg );
			break;

		default:
			UnknownRequest( tag, &msg );
			break;
		}
		
		if( ! L4_IsNilThread( TID ) )
		{
		    L4_Set_Label( &msg, L4_Label(tag) | 0x8000 );
		    L4_Load(&msg);
		    tag = L4_Send(TID);
		}
	}
}
