/* 
    Copyright Aleksey Mandrykin 2004-2007 
    This file is a part of Xameleon
*/
#include <sys/xam_console.h>

#include <l4/bootinfo.h>
#include <l4/kdebug.h>

#include <l4/types.h>
#include <l4/ipc.h>
#include <l4/kip.h>

//#include "include/xam_common.def"
#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>
#include <utime.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/device.h>
#include <mntent.h>

#include <string.h>
#include <sys/mount.h>
#include <stub.h>

#include <xam_requests.h>
#include <xam_errno.h>

#include <sys/termios.h>

#define ARRAY_SIZE(x)				(sizeof(x)/sizeof(x[0]))

static	bool						bShellRunning;
static 	struct termios				static_tms;
static	char						buffer[128];

extern L4_ThreadId_t				rootserver_id;
extern L4_KernelInterfacePage_t	*	kip;
extern L4_Word_t				*	env_size;

///////////////////////////////////////////////////////////////////////////////
// Callback's prototypes
//
int tcmdExit				( int argc, char * argv[], char * envp[] );
int tcmdHelp				( int argc, char * argv[], char * envp[] );
int tcmdReset				( int argc, char * argv[], char * envp[] );	// reset consoel
int tcmdKillProcess			( int argc, char * argv[], char * envp[] );
int tcmdMount				( int argc, char * argv[], char * envp[] );
int tcmdUnmount				( int argc, char * argv[], char * envp[] );
int tcmdChangeDir			( int argc, char * argv[], char * envp[] );
int tcmdShowDirectory		( int argc, char * argv[], char * envp[] );
int tcmdListDirectory		( int argc, char * argv[], char * envp[] );
int tcmdChangeRootDirectory	( int argc, char * argv[], char * envp[] );
int tcmdShowFile			( int argc, char * argv[], char * envp[] );
int tcmdRemoveFile			( int argc, char * argv[], char * envp[] );
int tcmdRemoveDirectory		( int argc, char * argv[], char * envp[] );
int tcmdCreateDirectory		( int argc, char * argv[], char * envp[] );
int tcmdMakeLink			( int argc, char * argv[], char * envp[] );
int tcmdMoveFile			( int argc, char * argv[], char * envp[] );
int tcmdDiskInformation		( int argc, char * argv[], char * envp[] );
int tcmdChangeOwner			( int argc, char * argv[], char * envp[] );
int tcmdFlushCaches			( int argc, char * argv[], char * envp[] );
int tcmdMakeDirectory		( int argc, char * argv[], char * envp[] );
int tcmdCopyFile			( int argc, char * argv[], char * envp[] );
int tcmdSetEnvironment		( int argc, char * argv[], char * envp[] );
int tcmdGetEnvironment		( int argc, char * argv[], char * envp[] );
int tcmdUnsetEnvironment	( int argc, char * argv[], char * envp[] );
int tcmdExec				( int argc, char * argv[], char * envp[] );
int tcmdPwd					( int argc, char * argv[], char * envp[] );
int tcmdRun					( int argc, char * argv[], char * envp[] );
int tcmdShell				( int argc, char * argv[], char * envp[] );
int tcmdEnv					( int argc, char * argv[], char * envp[] );
int tcmdDate				( int argc, char * argv[], char * envp[] );
int tcmdForkProcess			( int argc, char * argv[], char * envp[] );
int tcmdTouch				( int argc, char * argv[], char * envp[] );
int tcmdDate				( int argc, char * argv[], char * envp[] );

///////////////////////////////////////////////////////////////////////////////
// Callback's type definition
//
typedef struct typeCommandEntry
{
	const char		*	szCmd;
	int					nLength;
	int					(*fHandler)( int argc, char * argv[], char * envp[] );
	const char		*	szHelpString;
} typeCommandEntry;

///////////////////////////////////////////////////////////////////////////////
// Callback's array
//
static typeCommandEntry	ImplementedCommandList[] =
{
	{ "kill",		0, tcmdKillProcess },
	{ "reset",		0, tcmdReset },
	{ "mount",		0, tcmdMount },
	{ "umount",		0, tcmdUnmount },
	{ "ls",			0, tcmdListDirectory },
	{ "chroot",		0, tcmdChangeRootDirectory },
	{ "cat",		0, tcmdShowFile },
	{ "rm",			0, tcmdRemoveFile },
	{ "rmdir",		0, tcmdRemoveDirectory },
	{ "mkdir",		0, tcmdMakeDirectory },
	{ "ln",			0, tcmdMakeLink },
	{ "mv",			0, tcmdMoveFile },
	{ "df",			0, tcmdDiskInformation },
	{ "chown",		0, tcmdChangeOwner },
	{ "sync",		0, tcmdFlushCaches },
	{ "cp",			0, tcmdCopyFile },
	{ "pwd",		0, tcmdPwd },
	{ "xash",		0, tcmdShell },
	{ "env",		0, tcmdEnv },
	{ "date",		0, tcmdDate },
	{ "touch",		0, tcmdTouch },
	{ "help",		0, tcmdHelp },
};

static typeCommandEntry	BuiltinCommandList[] =
{
	{ "exit",		0, tcmdExit },
	{ "quit",		0, tcmdExit },
	{ "help",		0, tcmdHelp },
	{ "cd",			0, tcmdChangeDir },
	{ "dir",		0, tcmdShowDirectory },
	{ "sync",		0, tcmdFlushCaches },
	{ "setenv",		0, tcmdSetEnvironment },
	{ "getenv",		0, tcmdGetEnvironment },
	{ "unsetenv",	0, tcmdUnsetEnvironment },
	{ "exec",		0, tcmdExec },
	{ "env",		0, tcmdEnv },
	{ "reset",		0, tcmdReset },
	{ "date",		0, tcmdDate },
	{ "fork",		0, tcmdForkProcess },
};

inline const char * LocalGetName(const char * str)
{
	const char * s = strrchr( str, '/' );
	return s ? s+1 : str;
}

///////////////////////////////////////////////////////////////////////////////
//
//	Prints login prompt
//
static void printf_prompt(void)
{
#if 1
	char	CurrentPath[512];
	if( 0 != getcwd( CurrentPath, sizeof(CurrentPath) ) )
	{
		printf( "%s$", CurrentPath );
	}
	else
#endif
	{
		printf( "\r\n@" );
	}
	fflush(stdout);
}

///////////////////////////////////////////////////////////////////////////////
//
//	Split arguments argv. Return arguments count
//
static int split_arguments(
	char			*	s,
	const int			nMaxSize,
	char			*	argvector[] )
{
	int 				i;

	if( ! s ) return XAM_E2BIG;

	for( i = 0; i < nMaxSize; i++)
	{
		// skip spaces (do not forget to implement screening)
		for( ; (*s == ' ' || *s == '\t' || *s == '\r' || *s == '\n' );  s++ )
		{
			*s = 0;
		}

		// check termination condition
		if( !*s ) break;

		// set argument
		argvector[ i ] = s;

		// find next argument
		s = strpbrk( s, " \t\r\n" );
		if(!s)
		{
			i++;
			break;
		}
		*s++ = 0;
	}
	
	if ( i >= nMaxSize ) return XAM_E2BIG;

	argvector[ i ] = 0;

	return i;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdRun
// Params	  : 
// Return	  : 
// Description:	Run program
//
int run_file	( char * argv[], char * envp[] )
{
	pid_t			pid;
	int				nStatus = -1;
	int				nExitCode;

	pid = fork();
	if( pid == 0 ) 
	{
		char		*	path;
		int				i = 0; 
		int				j = 0;
		char			pathbuff[180];

		// Check if path is relative
		if( strrchr( argv[0], '/' ) )
		{
			nStatus = execve( argv[0], argv, envp );
		}
		else if( 0 != (path = getenv( "PATH" ) ) )
		{
			do {
				if( path[i] == ':' || path[i] == '\0' )
				{
					pathbuff[j++] = '/';
					pathbuff[j] = 0;
					strcat( pathbuff, argv[0] );
//					xam_printf( 0, "Runnning %s\n", pathbuff);
					nStatus = execve( pathbuff, argv, envp );
					j = 0;
				}
				else
				{
					pathbuff[j++] = path[i];
				}
			} while( path[i++] );
		}
		else nStatus = XAM_ENOENT;
		exit(XAM_ENOENT);
	}
	else if (pid < 0)
	{
		xam_printf( 0, "Unable fork - not enough resources\n");
		L4_KDB_Enter("BUG!!!");
		nStatus = pid;
	}
	else
	{
		nStatus = waitpid(pid, &nExitCode, 0);
//		xam_printf( 0, "Child exit with code %d,%d => Sending flush to parent\n", nStatus, nExitCode );
		if(nStatus >= 0) nStatus = nExitCode;
		
		ioctl ( 0, TIOCFLUSH, (char *) 0);
		ioctl ( 0, TIOCSETA, &static_tms );
	}

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
#define NO_MOUNT_TYPE	(const char*) 0
#define NO_MOUNT_DATA	(void*) 0
//
extern "C" int main( int argc, char * argv[], char * envp[] )
{
	unsigned int	nCmdIdx;
	int				nLength;
	int				nCommandLength;
	int				nResult;
	char		*	exe_name;

	// For debug libc we need next line
	xam_console_init("vty");

	// Initialize command list
	for( nCmdIdx = 0; nCmdIdx < ARRAY_SIZE(ImplementedCommandList); nCmdIdx++ ) 
		ImplementedCommandList[nCmdIdx].nLength = strlen( ImplementedCommandList[nCmdIdx].szCmd );

	/* Emulate busybox here */
	exe_name = strrchr( argv[0], '/' );
	if( ! exe_name ) exe_name = argv[0]; else exe_name++;
	nCommandLength = strlen(exe_name);

	for( nCmdIdx=0 ; nCmdIdx < ARRAY_SIZE(ImplementedCommandList); nCmdIdx++)
	{
		nLength = ImplementedCommandList[nCmdIdx].nLength;
		if( nLength != nCommandLength ) 
		{
			continue;
		}
		if( strcmp(ImplementedCommandList[nCmdIdx].szCmd, exe_name ) != 0 ) 
		{
			continue;
		}
		if( ImplementedCommandList[nCmdIdx].fHandler)
		{
			nResult = ImplementedCommandList[nCmdIdx].fHandler( argc, argv, envp );
			exit(nResult);
		}
	}
	printf( "Command %s does not implemented\n", exe_name);
	exit( XAM_ENOSYS );
	L4_KDB_Enter("We must never reach this point");
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//	Shell
//
int tcmdShell				( int argc, char * argv[], char * envp[] )
{
	unsigned int		nCmdIdx;
	pid_t			control_pid;
	int					nCommandLength;
	int					nResult = 0;

	unsigned int		nAgumentCount;
	char			*	argvector[12];

	printf( "\033[0mXameleon pseudoshell version 0.3\r\n" );

	for( nCmdIdx = 0; nCmdIdx < ARRAY_SIZE(BuiltinCommandList); nCmdIdx++ ) 
		BuiltinCommandList[nCmdIdx].nLength = strlen( BuiltinCommandList[nCmdIdx].szCmd );

	// Get and set controllling terminal
	control_pid = tcgetpgrp( 0 );
	tcsetpgrp( 0, getpid() );
	
	// Set backspace character
	ioctl(0, TIOCGETA, &static_tms );
	static_tms.c_cc[VERASE] = 0x08;
	ioctl(0, TIOCSETA, &static_tms );

	// Increase shell level
	{
		char * shlvl = getenv("SHLVL");
		if( ! shlvl ) setenv("SHLVL", "1", 1); else
		{
			char	sznum[16];
			nResult = atoi(shlvl);
			nResult++;
			snprintf(sznum, 15, "%d", nResult);
			setenv("SHLVL", sznum, 1);
		}
	}

	// Main application loop
	for(bShellRunning = true; bShellRunning;)
	{
		printf_prompt();

		nCommandLength = _gets( sizeof(buffer)-1, buffer );
		buffer[nCommandLength] = 0;

		// Split arguments
		nAgumentCount = split_arguments( buffer, ARRAY_SIZE(argvector), argvector );
		if( nAgumentCount <= 0 ) continue;

		for( nCmdIdx=0; nCmdIdx < ARRAY_SIZE(BuiltinCommandList); nCmdIdx++ )
		{
			if( strcmp( BuiltinCommandList[nCmdIdx].szCmd, argvector[0] ) ) continue;

			if( ! BuiltinCommandList[nCmdIdx].fHandler )
			{
				printf( "Command matched but handler is not defined!!!\n");
				break;
			}

			nResult = BuiltinCommandList[nCmdIdx].fHandler( nAgumentCount, argvector, envp );
			break;
		}

		if( nCmdIdx == ARRAY_SIZE(BuiltinCommandList) )
		{
			nResult = run_file( argvector, environ );
		}
		if( nResult )
		{
			printf( "%s: return error %d\n", buffer, nResult);
		}
	}
	printf("\nXameleon shell exiting\n");
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
int tcmdHelp( int argc, char * argv[], char * envp[] )
{
	unsigned int i;

	printf( "\033[32mThis is Xameleon toolbox.\033[0m\n" );
	printf( "Toolbox provides following commands into single executable:\n");

	for( i = 0; i<ARRAY_SIZE(ImplementedCommandList); i++ ) 
	{
		printf( "%-20s", ImplementedCommandList[i].szCmd );
	}
/*
	printf( "\nBuiltin commands are:\n");

	for( unsigned int i = 0; i<ARRAY_SIZE(BuiltinCommandList); i++ ) 
	{
		printf( "%-16s", BuiltinCommandList[i].szCmd );
	}
*/
	printf("\n");
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
int tcmdReset( int argc, char * argv[], char * envp[] )
{
	printf( "%cc", 27);
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
//
//
int tcmdExit( int argc, char * argv[], char * envp[] )
{
	bShellRunning = false;
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdKillProcess( int argc, char * argv[], char * envp[] )
{
	switch( argc )
	{
//		case 2:	return kill( L4_GlobalId( atoi( argv[1] ), 1).raw, 9 );	
//		case 3:	return kill( L4_GlobalId( atoi( argv[2] ), 1).raw, atoi( argv[1] ) );	
		case 2:	return kill( atoi( argv[1] ), 9 );	
		case 3:	return kill( atoi( argv[2] ), atoi( argv[1] ) );	
	}
	printf( "kill [signum] pid\n");
	return XAM_EINVAL;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdMount( int argc, char * argv[], char * envp[] )
{
	int			nStatus;

	do {
		if( argc == 1 )
		{
#if 0
			argv[1] = "/dev/files/mtab";
			nStatus = tcmdShowFile( 2, argv, envp ) ;
#else
			FILE * f1 = setmntent("/etc/mtab", "rw");
			if( f1 )
			{
				struct mntent *e;
				
				while( NULL != (e = getmntent (f1)) )
				{
					printf( "'%s' on '%s' type %s (%s)\n", 
						e->mnt_fsname, 
						e->mnt_dir, 
						e->mnt_type,
						e->mnt_opts);
				}
				endmntent(f1);
			}
#endif
			continue;
		}
		if( argc >= 3 )
		{
			nStatus = mount( argv[1], argv[2], argv[3], 0, NO_MOUNT_DATA);
			continue;
		}

		nStatus = XAM_EINVAL;
		printf( "Use:\nmount - to see current mounts\nmount source target type - mount new device\n");

	} while (false);

	return  nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdUnmount( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return umount(argv[1]);
	}
	printf( "Use: umount point\n");
	return  XAM_EINVAL;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdChangeDir( int argc, char * argv[], char * envp[] )
{
	if( argc == 1 )
	{
		char	*s  = getenv("HOME");
		if( s ) return chdir( s );
		else	printf( "Variable HOME is not defined\n");

	}
	else if( argc == 2 )
	{
		return  chdir( argv[1] );
	}
	printf( "Use: cd path\n");
	return  XAM_EINVAL;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdShowDirectory( int argc, char * argv[], char * envp[] )
{
	int					Status;
	typeDirentry	*	entry;
	DIR				*	dir;

	do {

		if( argc == 1 ) 
			dir = opendir( "." ); 
		else 
			dir = opendir( argv[1] ); 

		if( ! dir )
		{
			printf( "Unable open directory\n");
			Status = errno;
			break;
		}

		for(;true;)
		{
			entry  = readdir( dir );
			if( !entry ) break;
			printf( "%-12s\t%8lu\n", entry->d_name, entry->d_ino);
		}

		Status = closedir( dir );

	} while(false);

	return Status;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdListDirectory( int argc, char * argv[], char * envp[] )
{
	int					nStatus = -100;
	typeDirentry	*	entry;
	DIR				*	dir;
	struct stat			StatusBuffer;
	char				FileNameBuffer[512];
	char				Desc[32];
	char				Owner[32];
	char				dch;

	do {

		if( argc == 1 ) dir = opendir( "." ); else dir = opendir( argv[1] ); 

		if( ! dir )
		{
			printf( "ls: Unable open directory\n");
			nStatus = -1;
			break;
		}

		for(;true;)
		{
			entry  = readdir( dir );
			if( !entry ) break;

			if( argc == 1 ) 
				snprintf(FileNameBuffer, sizeof(FileNameBuffer), "%s", entry->d_name );
			else
				snprintf(FileNameBuffer, sizeof(FileNameBuffer), "%s/%s", argv[1], entry->d_name );

			nStatus = lstat(FileNameBuffer, &StatusBuffer );

			if( nStatus )
			{
				dch = '?';
				snprintf(Desc, sizeof(Desc), "   ?????");
			}
			else if( S_ISDIR( StatusBuffer.st_mode ) ) 
			{
				dch = '/';
				snprintf(Desc, sizeof(Desc), "   <DIR>");
			}
			else if( S_ISCHR( StatusBuffer.st_mode ) )
			{
				L4_ThreadId_t	t;
				dch = '-';
				t.raw = StatusBuffer.st_dev >> 32;
				snprintf( Desc, sizeof(Desc), "%3d,%3d", (unsigned int) L4_ThreadNo(t),  (unsigned int) StatusBuffer.st_dev );
			}
			else if( S_ISBLK( StatusBuffer.st_mode ) )
			{
				L4_ThreadId_t	t;
				dch = '+';
				t.raw = StatusBuffer.st_dev >> 32;
				snprintf( Desc, sizeof(Desc), "%3d,%3d", (unsigned int) L4_ThreadNo(t),  (unsigned int) StatusBuffer.st_dev );
			}
			else if( S_ISLNK( StatusBuffer.st_mode ) )
			{
				dch = '@';
				snprintf(Desc, sizeof(Desc), "<symlink>");
/*
				int fd = local_file_system.Open( pShell->m_pProcess, szFileName, O_NOLINK, 0 );
				if(fd >= 0)
				{
					int	sz;
					sz = local_file_system.Read( pShell->m_pProcess, fd, szSize, sizeof(szSize));
					local_file_system.Close( pShell->m_pProcess, fd );
					if(sz > 0) 
					{
						szSize[sz] = 0;
						printf("@%-12s -> %s\n", szFileName, szSize);
						break;
					}
				}
*/
			}
			else if( S_ISFIFO( StatusBuffer.st_mode ) )
			{
				dch = ' ';
				snprintf( Desc, sizeof(Desc), "%8s","|" );
			}
			else
			{
				dch = ' ';
				snprintf( Desc, sizeof(Desc), "%8d", (unsigned int) StatusBuffer.st_size );
			}

			snprintf(Owner, sizeof(Owner), "(%ld:%ld)", StatusBuffer.st_uid, (unsigned long) StatusBuffer.st_gid );

#if 1
			printf( "%c%-14s%8s %12s %06o %2d %5d\n", 
				dch,
				entry->d_name,
				Desc,
				Owner, 
				StatusBuffer.st_mode,
				StatusBuffer.st_nlink,
				StatusBuffer.st_ino
				);
#else
			
int c = snprintf( FileNameBuffer, sizeof(FileNameBuffer), "%c%-14s%8s %12s %06o %2d %5d\n", 
				dch,
				entry->d_name,
				Desc,
				Owner, 
				StatusBuffer.st_mode,
				StatusBuffer.st_nlink,
				StatusBuffer.st_ino
				);
	write( 1, FileNameBuffer, c );
#endif
		}

		if( !nStatus ) 
			nStatus = closedir( dir ); 
		else 
			closedir( dir );

	} while(false);

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdChangeRootDirectory	( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return  chroot( argv[1] );
	}
	printf("Use: chroot path\n");
	return  -1;
}

int cat_single_descriptor( int nFileDescriptor )
{
	int				nStatus = 0;
	int				nOffset;
	char			buff[1024];

	while(true)
	{
		nOffset = read( nFileDescriptor, buff, (size_t) sizeof(buff));
		if(nOffset < 0)
		{
			nStatus = nOffset;
//			fprintf( stderr, "Unable read file: %s error %d\n", argv[nArgIdx], nOffset );
			break;
		}
		if(nOffset == 0) break;
		
		// Write to stdout
		for( int i=0; nOffset; i++, nOffset-- ) printf( "%c", buff[i] );
	}
	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdShowFile ( int argc, char * argv[], char * envp[] )
{
	int				nStatus = 0;
	int				nFileDescriptor;
//	int				nOffset;
	int				nArgIdx;
//	char			buff[512];

	if( argc == 1 )
	{
		cat_single_descriptor( 0 ); // output stdin
	}
	else for ( nArgIdx = 1; nArgIdx < argc; nArgIdx++ )
	{
		if( argv[nArgIdx][0] != '-' ) 
		{
			nFileDescriptor = open( argv[nArgIdx], O_RDONLY, 0);
			if( nFileDescriptor < 0 )
			{
				nStatus = nFileDescriptor;
				printf( "Unable open file %s\n", argv[nArgIdx] );
				break;
			}
		}
		else switch( argv[nArgIdx][1] )
		{
			case 0: nFileDescriptor = 0; break;
			default: continue; // skip options 
		}

		cat_single_descriptor( nFileDescriptor ); // output stdin

		if( !nStatus ) 
			nStatus = close( nFileDescriptor );
		else
			close( nFileDescriptor );
		
	}

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdRemoveFile			( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return  unlink( argv[1] );
	}
	printf( "Use: rm filename\n");
	return  -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdRemoveDirectory		( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return  rmdir( argv[1] );
	}
	printf("Use: rmdir dirname\n");
	return  -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdMakeDirectory		( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return  mkdir( argv[1], 0777 );
	}
	printf( "Use: mkdir dirname\n");
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdCopyFile			( int argc, char * argv[], char * envp[] )
{
	int					nStatus;
	int					nSrcHandle;
	int					nDstHandle;

	int					nBytesRead;
	int					nBytesWrite;

	struct stat			SourceFileStatus;
	struct stat			DestinationFileStatus;
	char				TmpNameBuffer[4096];

	if(argc < 3)
	{
		printf( "Use: cp srcfile dst_file\n");
		return -1;
	}
	
	for(int j=0;j<argc;j++) xam_printf(0, "%s ",argv[j]); xam_printf(0, "\n");

	// Get file status of the last command's argument
	nDstHandle = stat( argv[ argc - 1 ], &DestinationFileStatus );

	if( nDstHandle == 0 && S_ISDIR( DestinationFileStatus.st_mode ) ) 
	{
		// Every source file name
		for( int i = 1; i < argc - 1; i++ )
		{
			// Open source file
			nSrcHandle = open( argv[i], O_RDONLY, 0 );
			if ( nSrcHandle < 0 ) return nSrcHandle;

			// Get file status of the first command's argument
			nStatus = fstat( nSrcHandle, &SourceFileStatus);
			if ( nStatus < 0 ) return nStatus;

			// Make destination file name
			snprintf( TmpNameBuffer, sizeof(TmpNameBuffer) - 1, "%s/%s", argv[ argc - 1 ], LocalGetName(argv[i]) );
			// Create destination file
			nDstHandle = creat( TmpNameBuffer, SourceFileStatus.st_mode & 0777 );
			if ( nDstHandle < 0 ) return nDstHandle;
			// Copy file
			while( (nBytesRead = read( nSrcHandle, TmpNameBuffer, (size_t) sizeof(TmpNameBuffer) )) > 0 )
			{
				nBytesWrite = write(  nDstHandle, TmpNameBuffer, (size_t) nBytesRead );
				if( nBytesWrite != nBytesRead ) return nBytesWrite;
			}
			close( nDstHandle );
			close( nSrcHandle );
		}
	}
	else
	{
		if( argc != 3 ) return XAM_EDOM;

		if ( nDstHandle == 0 && ! S_ISCHR( DestinationFileStatus.st_mode ) && ! S_ISFIFO( DestinationFileStatus.st_mode ) )
		{
			printf( "File already exist\n");
			close( nSrcHandle );
			return -1;
		}

		// Open source file
		nSrcHandle = open( argv[ 1 ], O_RDONLY, 0 );
		if ( nSrcHandle < 0 ) 
		{
			return nSrcHandle;
		}

		nStatus = fstat( nSrcHandle, &SourceFileStatus);
		if ( nStatus < 0 ) return nStatus;

		// We don't use exclusive create for case of character device or fifo
		nDstHandle = open( argv[ argc - 1 ], O_CREAT | /*O_EXCL |*/ O_WRONLY, SourceFileStatus.st_mode & 0777 );
		if ( nDstHandle < 0 ) 
		{
			printf( "Unable opend destionation file\n");
			close( nSrcHandle );
			return nDstHandle;
		}

//		nDstHandle = creat( argv[ argc - 1 ], SourceFileStatus.st_mode );
//		if ( nDstHandle < 0 ) 
//		{
//			close( nSrcHandle );
//			return nDstHandle;
//		}

		// Copy file
		while( (nBytesRead = read( nSrcHandle, TmpNameBuffer, (size_t) sizeof(TmpNameBuffer) )) > 0 )
		{
			nBytesWrite = write(  nDstHandle, TmpNameBuffer, nBytesRead );
			if( nBytesWrite != nBytesRead ) 
			{
				printf( "Error writing file (h:%d sz:%d): %d\n", nDstHandle, nBytesRead, nBytesWrite );
				close( nDstHandle );
				close( nSrcHandle );
				return nBytesWrite;
			}
		}
		close( nDstHandle );
		close( nSrcHandle );
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdCreateDirectory		( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return  mkdir( argv[1], 0700 );
	}
	printf( "Use: mkdir dirname\n");
	return  -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdMakeLink
// Params	  : 
// Return	  : 
// Description:	makes soft or hard link
//
int tcmdMakeLink	( int argc, char * argv[], char * envp[] )
{
	char			*	names[2];
	int					name_index = 0;
	bool				bOptionsError = false;
	bool				bHardLink = true;
	bool				bInteractive = false;
	bool				bForce = false;

	for( int i = 1; ! bOptionsError && i < argc; i++ )
	{
		if( argv[i][0] == '-' ) switch(argv[i][1])
		{
			case 's':	bHardLink = false;		break;
			case 'f':	bForce = true;			break;
			case 'i':	bInteractive = false;	break;
			default:	bOptionsError = true;	break;
		}
		else
		{
			if( name_index < 2 ) 
				names[name_index++] = argv[i]; 
			else 
				bOptionsError = true;
		}
	}

	if( ! bOptionsError || name_index == 2 )
	{
		if( bHardLink )
		{
			return link( names[0], names[1] );
		}
		else
		{
			return symlink( names[0], names[1] );
		}
	}
	else
	{
		printf("Use: ln [options] source dest\n");
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdMoveFile
// Params	  : 
// Return	  : 
// Description:	Moves or renames file[s]
//
int tcmdMoveFile			( int argc, char * argv[], char * envp[] )
{
	int					nStatus;
	char			*	names[2];
	int					name_index = 0;
	bool				bOptionsError = false;

	for( int i = 1; ! bOptionsError && i < argc; i++ )
	{
		if( argv[i][0] == '-' ) switch(argv[i][1])
		{
			default:	bOptionsError = true;	break;
		}
		else
		{
			if( name_index < 2 ) 
				names[name_index++] = argv[i]; 
			else 
				bOptionsError = true;
		}
	}

	if( ! bOptionsError || name_index == 2 )
	{
		struct stat			DestinationStatus;
		nStatus = stat( names[1], &DestinationStatus );
		if( nStatus ) 
		{
			nStatus = rename( names[0], names[1] );
		}
		else if( S_ISDIR( DestinationStatus.st_mode ) ) 
		{
			char	newname[256];
			char	*	p = strrchr( names[0], '/' );
			if( !p ) p = names[0]; else p++;
			snprintf( newname, sizeof(newname), "%s/%s", names[1], p );
			nStatus = rename( names[0], newname );
			printf("Moving : %s to %s\n", names[0], newname );
		}
	}
	else
	{
		printf("Use: mv [options] source dest\n");
		nStatus = -1;
	}
	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdDiskInformation
// Params	  : 
// Return	  : 
// Description:	Prints filesystem information
//
static int show_superblock_status( char * szMountPoint )
{
	int						nStatus;
	struct statfs			SuperblcokStatus;

	nStatus = statfs( szMountPoint, &SuperblcokStatus );
	if( nStatus == 0 )
	{
		printf( "%-32s 0x%04x %6d %6d %6d %6d\n", 
			szMountPoint,
			SuperblcokStatus.f_type,
			SuperblcokStatus.f_blocks,
			SuperblcokStatus.f_bfree,
			SuperblcokStatus.f_files,
			SuperblcokStatus.f_ffree);
	}
	else
	{
		printf( "%s: %s\n", szMountPoint, strerror(nStatus) );
	}
	return nStatus;
}

int tcmdDiskInformation		( int argc, char * argv[], char * envp[] )
{
	int						nStatus = XAM_EFAULT;
	struct statfs			SuperblcokStatus;
	char					CurrentPath[512];

	char				*	szMountPoint;

	printf( "Mount point                       Type   Blocks FreeBl Files  FreeFl\n" 
			 "--------------------------------- ------ ------ ------ ------ ------\n" );

	if( argc == 1 ) 
	{
		char				line[128];

		FILE * f = fopen("/etc/mtab", "rt" );
		if( f )
		{
			while( ! feof(f) )
			{
				if( ! fgets( line, sizeof(line), f ) ) break;
				for( int len = 34; line[len]; len++ ) 
				{
					if( line[len] == '#' || line[len] == ' ' || line[len] == '\t' )
					{
						line[len] = '\0';
						break;
					}
				}
//				printf("Read line: '%s'\n", line);
				szMountPoint = line + 34;
				nStatus = show_superblock_status( szMountPoint );
				if( 0 > nStatus ) break;
			}
			fclose(f);
		}
	}
	else
	{
		szMountPoint = argv[1];
		if( strcmp(szMountPoint , "." ) == 0 )
		{
			if( 0 != getcwd( CurrentPath, sizeof(CurrentPath) ) )
			{
				szMountPoint = CurrentPath;
			}
		}
		nStatus = show_superblock_status( szMountPoint );
	}

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdChangeOwner
// Params	  : 
// Return	  : 
// Description:	Changes a file, symlink,r directory, or device owner
//
int tcmdChangeOwner			( int argc, char * argv[], char * envp[] )
{
	if( argc == 3 )
	{
		// To fix: it does reset group to zero.
		return chown( argv[2], atoi(argv[1]), 0 );
	}
	else
	{
		printf( "Use: chown numeric_id path\n");
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdSetEnvironment
// Params	  : 
// Return	  : 
// Description:	Set value of ewnvironment variable
//
int tcmdSetEnvironment		( int argc, char * argv[], char * envp[] )
{
	if( argc == 3 )
	{
		return setenv( argv[1], argv[2], 1 );
	}
	else
	{
		printf( "Use: setenv name value\n");
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdGetEnvironment
// Params	  : 
// Return	  : 
// Description:	Get value of ewnvironment variable
//
int tcmdGetEnvironment		( int argc, char * argv[], char * envp[] )
{
//	char		value_buffer[128];

	printf( "argc %d : %p %p %p\n", argc, argv[0], argv[1], argv[2] );
	if( argc == 2 )
	{
		char * value = getenv( argv[1] );
		if( value )
		{
			printf( "%s = %s\n", argv[1], value);
			return 0;
		}
		else
			printf( "Variable %s is not difened\n", argv[1] );
	}
	else
	{
		printf( "Use: getenv name\n");
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdUnsetEnvironment
// Params	  : 
// Return	  : 
// Description:	Delete an environment variable
//
int tcmdUnsetEnvironment	( int argc, char * argv[], char * envp[] )
{
	if( argc == 2 )
	{
		return unsetenv( argv[1] );
	}
	else
	{
		printf( "Use: unsetenv name\n");
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdExec
// Params	  : 
// Return	  : 
// Description:	Executes program
//
int tcmdExec				( 
	int				argc, 
	char	*		argv[], 
	char	*		envp[] )
{
	if( argc < 2 )
	{
		printf( "Use: exec name\n");
		return -1;
	}

	return execve( argv[1], &argv[1], environ );
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdChangeOwner
// Params	  : 
// Return	  : 
// Description:	Changes a file, symlink,r directory, or device owner
//
int tcmdFlushCaches			( int argc, char * argv[], char * envp[] )
{
	return sync();
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : tcmdPwd
// Params	  : 
// Return	  : 
// Description:	Print current directory name
//
int tcmdPwd					( 
	int				argc, 
	char	*		argv[], 
	char	*		envp[] )
{
	int		nStatus = -1;
	char	CurrentPath[512];

	if( 0 != getcwd( CurrentPath, sizeof(CurrentPath) ) )
	{
		printf( "%s", CurrentPath );
		nStatus = 0;
	}
	return nStatus;
}

extern L4_Word_t * env_size;

int tcmdEnv	( int argc, char * argv[], char * envp[] )
{
	if( envp ) for( int i=0; environ[i]; i++ )
	{
		printf( "%s\n", environ[i]);
	}
	else
	{
		printf( "No environment set by exec process. env_size is %d on %p\n", *env_size, env_size );
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdForkProcess( int argc, char * argv[], char * envp[] )
{
	int					nStatus;
	struct stat			StatusBuffer;
	pid_t				pid;

	do {

		if(argc != 2)
		{
			printf( "Use: fork /dev/vty[12345]\n");
			nStatus = -1;
			break;
		}

		nStatus = stat( argv[1], &StatusBuffer );
		if( nStatus )
		{
			printf( "File not found\n");
			break;
		}

		if( ! S_ISCHR( StatusBuffer.st_mode ) )
		{
			printf( "%s is not character device\n", argv[1] );
			nStatus = -1;
			break;
		}

		pid = fork();
		if( pid == 0 ) 
		{
			ioctl ( 0, TIOCFLUSH, (char *) 0);
			close(0);	// close stdin
			close(1);	// close stdout
			close(2);	// close stderr

			if( 0 != open( argv[1], O_RDONLY, 0) ) // open stdin
			{
				xam_printf(0, "Unable open standard input stream\n");
			}
			if( 1 != open( argv[1], O_WRONLY, 0) )	// open stdout
			{
				xam_printf(0, "Unable open standard output stream\n");
			}
			if( 2 != open( argv[1], O_WRONLY, 0) )	// open stderr
			{
				xam_printf(0, "Unable open standard error stream\n");
			}
		}
	
	} while( false);

	return 0;
}
///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//
int tcmdTouch				( int argc, char * argv[], char * envp[] )
{
	int				nStatus;
	utimbuf_t		timebuf;

	timebuf.modtime = timebuf.actime = time( NULL );

	do {

		nStatus = utime( argv[1], &timebuf );
		if( nStatus < 0 )
		{
			printf("utme status %d\n", nStatus );
			nStatus = creat( argv[1], O_RDWR );
			if( nStatus < 0 ) 
			{
				printf("ceaat status %d\n", nStatus);
				break;
			}
			nStatus = close( nStatus );
			if( nStatus ) 
			{
				printf("close status %d\n", nStatus);
				break;
			}
		}
	} while( false );

	return nStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Name		  : 
// Params	  : 
// Return	  : 
// Description:	
//

#if 1

int tcmdDate				( int argc, char * argv[], char * envp[] )
{
	time_t			t;

	t = time(NULL);

	puts( ctime(&t) );

	return 0;
}

#else

int tcmdDate		( int argc, char * argv[], char * envp[] )
{
	struct	tm	*	t;
	time_t			tt;

	tt = time(0);
	t = localtime( &tt );

	printf( "Now is %d:%02d:%02d %d.%02d.%02d = %d sec\n", 
		t->tm_hour, t->tm_min, t->tm_sec, t->tm_year, t->tm_mon, t->tm_mday, tt );

	return 0;
}

#endif
